#include <sys/types.h>
#include <signal.h>

#ifdef POSIX_SIGNALS

/*********************************************************************/
/* Function: mysignal()
/*  POSIX style signals.
/*
/*  signal() has different semantics over different versions of unix.
/*  this emulates signal() with sigaction() to behave like BSD.
/*
/* From Stevens Advanced Programming in the UNIX environment */
/*
/*********************************************************************/
void *mysignal(signo, func)
  int signo;
  void *func;

{
  struct sigaction act, oact;
  
  act.sa_handler = (void*)func;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;

  if (signo == SIGALRM) {
#ifdef  SA_INTERRUPT
  act.sa_flags |= SA_INTERRUPT; /* SunOS */
#endif
  } else {
#ifdef SA_RESTART
  act.sa_flags |= SA_RESTART; /* SVR4, 4.3+BSD */
#endif
  }

  if (sigaction(signo, &act, &oact) < 0)
    return SIG_ERR;

  return oact.sa_handler;
} /* signal */

#endif /* POSIX_SIGNALS */

